@@ -14,6 +14,10 @@ class IntegrityError(WebAuthnRPError):
   pass
 
 
+class OriginError(WebAuthnRPError):
+  pass
+
+
 class VerificationError(WebAuthnRPError):
   pass
 
@@ -34,6 +38,10 @@ class ValidationError(WebAuthnRPError):
   pass
 
 
+class BuilderError(WebAuthnRPError):
+  pass
+
+
 class NotFoundError(WebAuthnRPError):
   pass
 
