@@ -7,13 +7,13 @@ from enum import Enum
 from functools import singledispatch
 from typing import Any, List, Optional, Sequence, Set, Tuple, Union, cast
 
-import cbor
+import cbor2
 
-from webauthn_rp.constants import (EC2_P_256_NUMBER_LENGTH,
-                                   EC2_P_384_NUMBER_LENGTH,
-                                   EC2_P_521_NUMBER_LENGTH,
-                                   OKP_ED448_NUMBER_LENGTH,
-                                   OKP_ED25519_NUMBER_LENGTH)
+from webauthn_rp.constants import (ED448_COORDINATE_BYTE_LENGTH,
+                                   ED25519_COORDINATE_BYTE_LENGTH,
+                                   P_256_COORDINATE_BYTE_LENGTH,
+                                   P_384_COORDINATE_BYTE_LENGTH,
+                                   P_521_COORDINATE_BYTE_LENGTH)
 from webauthn_rp.errors import (DecodingError, TokenBindingError,
                                 ValidationError)
 from webauthn_rp.types import (
@@ -21,15 +21,16 @@ from webauthn_rp.types import (
     AttestationObject, AttestationStatement,
     AttestationStatementFormatIdentifier, AttestedCredentialData,
     AuthenticationExtensionsClientOutputs, AuthenticatorAssertionResponse,
-    AuthenticatorAttestationResponse, AuthenticatorData, CollectedClientData,
-    Coordinates, COSEAlgorithmIdentifier, COSEKeyOperation, COSEKeyType,
-    CredentialPublicKey, EC2CredentialPublicKey, EC2KeyType,
-    FIDOU2FAttestationStatement, NoneAttestationStatement,
-    OKPCredentialPublicKey, OKPKeyType, PackedAttestationStatement,
+    AuthenticatorAttestationResponse, AuthenticatorData, AuthenticatorDataFlag,
+    CollectedClientData, Coordinates, COSEAlgorithmIdentifier,
+    COSEKeyOperation, COSEKeyType, CredentialPublicKey, EC2CredentialPublicKey,
+    EC2Curve, FIDOU2FAttestationStatement, NoneAttestationStatement,
+    OKPCredentialPublicKey, OKPCurve, PackedAttestationStatement,
     PackedECDAAAttestationStatement, PackedX509AttestationStatement,
     PublicKeyCredential, TokenBinding, TokenBindingStatus,
     TPMAttestationStatement, TPMECDAAAttestationStatement,
     TPMX509AttestationStatement)
+from webauthn_rp.utils import curve_coordinate_byte_length
 from webauthn_rp.validators import validate
 
 
@@ -66,13 +67,6 @@ def bytes_from_base64(s: str) -> bytes:
     raise ValidationError('Invalid base64 string')
 
 
-def bytes_from_array(arr: Sequence[Any]) -> bytes:
-  try:
-    return bytes(arr)
-  except binascii.Error:
-    raise ValidationError('Invalid array type')
-
-
 def parse_public_key_credential(data: dict) -> PublicKeyCredential:
   check_unsupported_keys({'id', 'rawId', 'response', 'type'}, data)
   id_ = parse_dictionary_field('id', str, data)
@@ -106,8 +100,8 @@ def parse_public_key_credential(data: dict) -> PublicKeyCredential:
         parse_dictionary_field('authenticatorData', str, response))
     signature = bytes_from_base64(
         parse_dictionary_field('signature', str, response))
-    user_handle_b64s = parse_dictionary_field('userHandle', (list, tuple),
-                                              response, False)
+    user_handle_b64s = parse_dictionary_field('userHandle', str, response,
+                                              False)
     user_handle = None
     if user_handle_b64s is not None:
       user_handle = bytes_from_base64(user_handle_b64s)
@@ -129,7 +123,7 @@ def parse_credential_public_key_kty(
   try:
     kty = cast(Union[COSEKeyType.Name, COSEKeyType.Value],
                COSEKeyType(kty_raw))  # type: ignore
-  except KeyError:
+  except (KeyError, ValueError):
     raise ValidationError(
         'Invalid credential public key type {}'.format(kty_raw))
   return kty
@@ -144,7 +138,7 @@ def parse_credential_public_key_alg(
     alg = cast(Union[COSEAlgorithmIdentifier.Name,
                      COSEAlgorithmIdentifier.Value],
                COSEAlgorithmIdentifier(alg_raw))  # type: ignore
-  except KeyError:
+  except (KeyError, ValueError):
     raise ValidationError(
         'Invalid credential public key alg type {}'.format(alg_raw))
   return alg
@@ -171,7 +165,7 @@ def parse_credential_public_key_key_ops(
 
     try:
       key_ops.append(COSEKeyOperation(ko))  # type: ignore
-    except KeyError:
+    except (KeyError, ValueError):
       raise ValidationError('Invalid credential public key key op {}'.format(
           credential_public_key[4]))
 
@@ -193,52 +187,28 @@ def parse_credential_public_key_kwargs(credential_public_key: dict) -> dict:
   }
 
 
-def ec2_number_length(crv: Union[EC2KeyType.Name, EC2KeyType.Value]) -> int:
-  key_to_length = {
-      EC2KeyType.Name.P_256: EC2_P_256_NUMBER_LENGTH,
-      EC2KeyType.Name.P_384: EC2_P_384_NUMBER_LENGTH,
-      EC2KeyType.Name.P_521: EC2_P_521_NUMBER_LENGTH,
-      EC2KeyType.Value.P_256: EC2_P_256_NUMBER_LENGTH,
-      EC2KeyType.Value.P_384: EC2_P_384_NUMBER_LENGTH,
-      EC2KeyType.Value.P_521: EC2_P_521_NUMBER_LENGTH,
-  }
-
-  return key_to_length[crv]
-
-
-def okp_number_length(crv: Union[OKPKeyType.Name, OKPKeyType.Value]) -> int:
-  key_to_length = {
-      OKPKeyType.Name.ED25519: OKP_ED25519_NUMBER_LENGTH,
-      OKPKeyType.Name.ED448: OKP_ED448_NUMBER_LENGTH,
-      OKPKeyType.Value.ED25519: OKP_ED25519_NUMBER_LENGTH,
-      OKPKeyType.Value.ED448: OKP_ED448_NUMBER_LENGTH,
-  }
-
-  return key_to_length[crv]
-
-
 def parse_ec2_public_key_crv(
-    credential_public_key: dict) -> Union[EC2KeyType.Name, EC2KeyType.Value]:
+    credential_public_key: dict) -> Union[EC2Curve.Name, EC2Curve.Value]:
   crv_raw = parse_dictionary_field(-1, (int, str), credential_public_key)
   try:
-    return EC2KeyType(crv_raw)  # type: ignore
-  except KeyError:
+    return EC2Curve(crv_raw)  # type: ignore
+  except (KeyError, ValueError):
     raise ValidationError('Invalid EC2 curve {}'.format(crv_raw))
 
 
 def parse_okp_public_key_crv(
-    credential_public_key: dict) -> Union[OKPKeyType.Name, OKPKeyType.Value]:
+    credential_public_key: dict) -> Union[OKPCurve.Name, OKPCurve.Value]:
   crv_raw = parse_dictionary_field(-1, (int, str), credential_public_key)
   try:
-    return OKPKeyType(crv_raw)  # type: ignore
-  except KeyError:
+    return OKPCurve(crv_raw)  # type: ignore
+  except (KeyError, ValueError):
     raise ValidationError('Invalid OKP curve {}'.format(crv_raw))
 
 
 def parse_okp_public_key(credential_public_key: dict) -> CredentialPublicKey:
   x = parse_dictionary_field(-2, bytes, credential_public_key)
   crv = parse_okp_public_key_crv(credential_public_key)
-  crv_len = okp_number_length(crv)
+  crv_len = curve_coordinate_byte_length(crv)
   if len(x) != crv_len:
     raise ValidationError(
         'Packed credential public key x and y must be {} bytes'.format(
@@ -256,7 +226,7 @@ def parse_ec2_public_key(
   x = parse_dictionary_field(-2, bytes, credential_public_key)
   y = parse_dictionary_field(-3, bytes, credential_public_key)
   crv = parse_ec2_public_key_crv(credential_public_key)
-  crv_len = ec2_number_length(crv)
+  crv_len = curve_coordinate_byte_length(crv)
   if len(x) != crv_len or len(y) != crv_len:
     raise ValidationError(
         'Packed credential public key x and y must be {} bytes'.format(
@@ -304,7 +274,7 @@ def parse_extensions(
       raise ValidationError('appid extension client output should be a bool')
 
   if tx_auth_simple is not None:
-    if type(tx_auth_simple) is not bool:
+    if type(tx_auth_simple) is not str:
       raise ValidationError(
           'tx_auth_simple extension client output should be a str')
 
@@ -334,7 +304,7 @@ def parse_extensions(
     if type(loc) is not dict:
       raise ValidationError('loc extension client output should be dict')
 
-    if any(type(x) is not float for x in loc.values()):
+    if any(type(x) not in (int, float) for x in loc.values()):
       raise ValidationError('Coordinate value in loc extension must be float')
 
     supported_cvalues = {
@@ -365,7 +335,7 @@ def parse_extensions(
             'uvm[{0}] extension client output should be list'.format(i))
 
       for j, v in enumerate(uvm_entry):
-        if type(v) is not str:
+        if type(v) is not int:
           raise ValidationError(
               'uvm[{0}][{1}] extension client output should be str'.format(
                   i, j))
@@ -395,7 +365,7 @@ def parse_attestation_statement_alg(
 
   try:
     alg = COSEAlgorithmIdentifier(alg)  # type: ignore
-  except KeyError:
+  except (KeyError, ValueError):
     raise ValidationError('Invalid algorithm identifier {}'.format(alg))
   return alg
 
@@ -522,6 +492,9 @@ def parse_client_data(
   except (UnicodeDecodeError, json.JSONDecodeError):
     raise DecodingError('Could not decode the client data JSON')
 
+  if type(client_data) is not dict:
+    raise ValidationError('Client data JSON must be a dictionary')
+
   type_ = client_data.get('type')
   challenge = client_data.get('challenge')
   origin = client_data.get('origin')
@@ -532,16 +505,28 @@ def parse_client_data(
   token_binding_data = client_data.get('tokenBinding')
   token_binding = None
   if token_binding_data is not None:
+    if type(token_binding_data) is not dict:
+      raise ValidationError('Token Binding data must be a dictionary')
+
     token_binding_status = token_binding_data.get('status')
     token_binding_id = token_binding_data.get('id')
 
-    if token_binding_status == TokenBindingStatus.PRESENT and (token_binding_id
-                                                               is None):
+    if token_binding_status is None:
+      raise ValidationError('Token Binding status must be present')
+
+    try:
+      token_binding_status_enum = TokenBindingStatus(token_binding_status)
+    except ValueError:
+      raise ValidationError(
+          'Invalid Token Binding status {}'.format(token_binding_status))
+
+    if token_binding_status_enum == TokenBindingStatus.PRESENT and (
+        token_binding_id is None):
       raise TokenBindingError(
           'Token Binding must contain an id if status is {}'.format(
               TokenBindingStatus.PRESENT))
 
-    token_binding = TokenBinding(status=token_binding_status,
+    token_binding = TokenBinding(status=token_binding_status_enum,
                                  id=token_binding_id)
 
   return CollectedClientData(type=type_,
@@ -553,20 +538,35 @@ def parse_client_data(
 def parse_cose_key(
     credential_public_key: Union[dict, bytes]) -> CredentialPublicKey:
   if type(credential_public_key) is bytes:
-    credential_public_key = cbor.loads(credential_public_key)
+    try:
+      credential_public_key = cbor2.loads(credential_public_key)
+    except cbor2.CBORDecodeError:
+      raise ValidationError('Could not decode credential public key CBOR')
 
-  cose_key_type = cast(Union[COSEKeyType.Name, COSEKeyType.Value],
-                       COSEKeyType(credential_public_key[1]))  # type: ignore
+    if type(credential_public_key) is not dict:
+      raise ValidationError('Credential public key CBOR must be a dictionary')
+  try:
+    cose_key_type = COSEKeyType(credential_public_key[1])  # type: ignore
+  except (KeyError, ValueError):
+    raise ValidationError('Invalid or missing COSE key type encountered')
 
   try:
-    cpk_parser = getattr(CredentialPublicKeyParser, cose_key_type.name)
+    cpk_parser = getattr(CredentialPublicKeyParser,
+                         cose_key_type.name)  # type: ignore
   except AttributeError:
     raise ValidationError('Parser not supported for key type {}'.format(
-        cose_key_type.name))
+        cose_key_type.name))  # type: ignore
 
   return cpk_parser(credential_public_key)
 
 
+def _read_bytes(bio: io.BytesIO, n: int) -> bytes:
+  x = bio.read1(n)
+  if len(x) != n:
+    raise EOFError('Unexpected number of bytes read')
+  return x
+
+
 def parse_authenticator_data(
     auth_data: bytes,
     fmt: Optional[AttestationStatementFormatIdentifier] = None
@@ -579,44 +579,62 @@ def parse_authenticator_data(
   signature_counter_bytes = auth_data[33:37]
   signature_counter_uint32, = struct.unpack('>I', signature_counter_bytes)
 
+  attested_credential_data_included = bool(flags
+                                           & AuthenticatorDataFlag.AT.value)
+  extension_data_included = bool(flags & AuthenticatorDataFlag.ED.value)
+
+  remaining_bytes_io = io.BytesIO(auth_data[37:])
+
   attested_credential_data = None
   aeci = None
-  if len(auth_data) >= 53:
-    aaguid = auth_data[37:53]
-    credential_id_length_bytes = auth_data[53:55]
-    credential_id_length_uint16, = struct.unpack('>H',
-                                                 credential_id_length_bytes)
-    credential_id = auth_data[55:(55 + credential_id_length_uint16)]
-    credential_public_key_bytes = auth_data[(55 +
-                                             credential_id_length_uint16):]
 
+  if attested_credential_data_included:
     try:
-      credential_public_key, bytes_read = cbor.cbor._loads(
-          io.BytesIO(credential_public_key_bytes))
-    except ValueError:
-      raise DecodingError('Could not decode the credential public key CBOR')
+      aaguid = _read_bytes(remaining_bytes_io, 16)
+      credential_id_length_bytes = _read_bytes(remaining_bytes_io, 2)
+      credential_id_length_uint16, = struct.unpack('>H',
+                                                   credential_id_length_bytes)
+      credential_id = _read_bytes(remaining_bytes_io,
+                                  credential_id_length_uint16)
 
-    if type(credential_public_key) is not dict:
-      raise ValidationError('Credential public key must be a dictionary')
-
-    cpk = parse_cose_key(credential_public_key)
-    validate(cpk)
-
-    extension_bytes = credential_public_key_bytes[bytes_read:]
-
-    if extension_bytes:
       try:
-        extensions = cbor.loads(extension_bytes)
-      except ValueError:
-        raise DecodingError('Could not decode the extensions CBOR')
-      aeci = parse_extensions(extensions)
+        credential_public_key = cbor2.load(remaining_bytes_io)
+      except cbor2.CBORDecodeError:
+        raise DecodingError('Could not decode the credential public key CBOR')
 
-    attested_credential_data = AttestedCredentialData(
-        aaguid=aaguid,
-        credential_id_length=credential_id_length_uint16,
-        credential_id=credential_id,
-        credential_public_key=cpk,
-    )
+      if type(credential_public_key) is not dict:
+        raise ValidationError('Credential public key must be a dictionary')
+
+      cpk = parse_cose_key(credential_public_key)
+      validate(cpk)
+
+      attested_credential_data = AttestedCredentialData(
+          aaguid=aaguid,
+          credential_id_length=credential_id_length_uint16,
+          credential_id=credential_id,
+          credential_public_key=cpk,
+      )
+    except EOFError:
+      raise ValidationError(
+          'Could not read the included attested credential data')
+
+  if extension_data_included:
+    try:
+      try:
+        extensions = cbor2.load(remaining_bytes_io)
+      except cbor2.CBORDecodeError:
+        raise DecodingError('Could not decode the extensions CBOR')
+
+      if type(extensions) is not dict:
+        raise ValidationError('Extension data CBOR must be a dictionary')
+
+      aeci = parse_extensions(extensions)
+    except EOFError:
+      raise ValidationError('Could not read the included extension data')
+
+  if remaining_bytes_io.read1(1) != b'':
+    raise ValidationError(
+        'The authenticator data has unexpected leftover bytes')
 
   return AuthenticatorData(
       rp_id_hash=rp_id_hash,
@@ -630,10 +648,13 @@ def parse_authenticator_data(
 def parse_attestation(
     attestation_object: bytes) -> Tuple[AttestationObject, dict]:
   try:
-    attestation_object_data = cbor.loads(attestation_object)
-  except ValueError:
+    attestation_object_data = cbor2.loads(attestation_object)
+  except cbor2.CBORDecodeError:
     raise DecodingError('Could not decode the attestation object CBOR')
 
+  if type(attestation_object_data) is not dict:
+    raise ValidationError('Attestation object CBOR must be a dictionary')
+
   try:
     auth_data = attestation_object_data['authData']
     fmt = attestation_object_data['fmt']
@@ -644,7 +665,7 @@ def parse_attestation(
 
     try:
       asfi = AttestationStatementFormatIdentifier(fmt)
-    except KeyError:
+    except ValueError:
       raise ValidationError('Invalid attestation statement format identifier')
 
     if type(auth_data) is not bytes:
@@ -652,7 +673,7 @@ def parse_attestation(
 
     authenticator_data = parse_authenticator_data(auth_data, asfi)
   except KeyError as e:
-    raise ValidationError('Missing {} in attestation'.format(str(e)))
+    raise ValidationError('Missing key in attestation ({})'.format(str(e)))
 
   if type(att_stmt) is not dict:
     raise ValidationError('attStmt must be a dictionary')
