@@ -79,11 +79,16 @@ class PublicKeyCredentialRpEntity(PublicKeyCredentialEntity):
       Authenticators MAY ignore an icon member’s value if its length is
       greater than 128 bytes. The URL’s scheme MAY be "data" to avoid fetches
       of the URL, at the cost of needing more storage.
-    id (str): A unique identifier for the Relying Party entity.
+    id (str): A unique identifier for the Relying Party entity. By default, the
+      RP ID for a WebAuthn operation is set to the caller’s origin's effective
+      domain. This default MAY be overridden by the caller, as long as the 
+      caller-specified RP ID value is a registrable domain suffix of or is 
+      equal to the caller’s origin's effective domain.
 
   References:
     * https://w3.org/TR/webauthn/#dictdef-publickeycredentialrpentity
     * https://w3.org/TR/webauthn/#dictdef-publickeycredentialentity
+    * https://www.w3.org/TR/webauthn/#rp-id
   """
   def __init__(self, *, name: str, icon: Optional[str] = None, id: str):
     super().__init__(name=name, icon=icon)
@@ -1214,9 +1219,9 @@ class COSEKeyOperation(metaclass=NameValueEnumsContainer):
     MAC_VERIFY = 10
 
 
-class EC2KeyType(metaclass=NameValueEnumsContainer):
+class EC2Curve(metaclass=NameValueEnumsContainer):
   """
-  A metaclass for the COSEKeyOperation Name and Value Enums.
+  A metaclass for the EC2Curve Name and Value Enums.
 
   Both the Name and Value Enums share the following documentation.
 
@@ -1239,9 +1244,9 @@ class EC2KeyType(metaclass=NameValueEnumsContainer):
     P_521 = 3
 
 
-class OKPKeyType(metaclass=NameValueEnumsContainer):
+class OKPCurve(metaclass=NameValueEnumsContainer):
   """
-  A metaclass for the COSEKeyOperation Name and Value Enums.
+  A metaclass for the OKPCurve Name and Value Enums.
 
   Both the Name and Value Enums share the following documentation.
 
@@ -1361,7 +1366,7 @@ class EC2CredentialPublicKey(CredentialPublicKey):
       algorithms lose security if the same IV is used twice.
     x (bytes): The Elliptic Curve X Coordinate.
     y (bytes): The Elliptic Curve Y Coordinate.
-    crv (Union[EC2KeyType.Name, EC2KeyType.Value]):
+    crv (Union[EC2Curve.Name, EC2Curve.Value]):
       The Elliptic Curve Identifier.
 
   References:
@@ -1383,7 +1388,7 @@ class EC2CredentialPublicKey(CredentialPublicKey):
       base_IV: Optional[bytes] = None,
       x: bytes,
       y: bytes,
-      crv: Union[EC2KeyType.Name, EC2KeyType.Value]):
+      crv: Union[EC2Curve.Name, EC2Curve.Value]):
     super().__init__(kty=kty,
                      kid=kid,
                      alg=alg,
@@ -1433,7 +1438,7 @@ class OKPCredentialPublicKey(CredentialPublicKey):
       algorithms lose security if the same IV is used twice.
     x (bytes): The Elliptic Curve X Coordinate.
     y (bytes): The Elliptic Curve Y Coordinate.
-    crv (Union[OKPKeyType.Name, OKPKeyType.Value]):
+    crv (Union[OKPCurve.Name, OKPCurve.Value]):
       The Elliptic Curve Identifier.
 
   References:
@@ -1453,7 +1458,7 @@ class OKPCredentialPublicKey(CredentialPublicKey):
       key_ops: Optional[Sequence[Union[COSEKeyOperation.Name,
                                        COSEKeyOperation.Value]]] = None,
       base_IV: Optional[bytes] = None,
-      crv: Union[OKPKeyType.Name, OKPKeyType.Value],
+      crv: Union[OKPCurve.Name, OKPCurve.Value],
       x: bytes):
     super().__init__(kty=kty,
                      kid=kid,
